import { execSync } from "node:child_process";
import { readdirSync } from "node:fs";
import prisma, { Prisma } from "./prisma";

const clearFailedMigrations = true;
const exitOnMigrationFailure = true;

async function getCurrentState() {
  try {
    const currentState: {
      id: string;
      checksum: string;
      finished_at?: Date;
      migration_name: string;
      logs: string | null;
      rolled_back_at: Date | null;
      started_at: Date;
      applied_steps_count: number;
    }[] = await prisma.$queryRaw`SELECT *
                                 FROM _prisma_migrations`;
    return currentState;
  } catch (e) {
    if (
      e instanceof Prisma.PrismaClientKnownRequestError &&
      e.meta?.code === "42P01"
    ) {
      console.log("No migrations table found, so we have a empty DB.");
      return [];
    } else throw e;
  }
}

export default async function migrate() {
  console.log("Checking if migrations are pending...");
  try {
    const currentState = await getCurrentState();
    console.log(currentState);
    if (currentState.find((state) => !state.finished_at)) {
      if (clearFailedMigrations) {
        console.log("Clearing failed migrations.");
        await prisma.$queryRaw`DELETE
                               FROM _prisma_migrations
                               WHERE finished_at IS NULL`;
      }
      throw new Error("A migration has failed before. Exiting.");
    }
    const files = readdirSync("prisma/migrations").filter(
      (item) => item != "migration_lock.toml",
    );
    if (files.length === currentState.length) {
      console.log("No pending migrations.");
      return;
    }
    console.log(
      `Found ${files.length - currentState.length} pending migrations.`,
    );
    // console.log("waiting 3 minutes for db to be stable...");
    // await new Promise((resolve) => setTimeout(resolve, 3 * 60 * 1000));
    console.log('Running "prisma migrate deploy"...');
    execSync("node_modules/.bin/prisma migrate deploy", { stdio: "inherit" });
  } catch (e) {
    console.error("Migrations failed:", e);
    if (exitOnMigrationFailure) process.exit(1);
    console.error("Continuing anyway... This might cause issues.");
  }
  console.log("Launching application...");
}
