import { Redis } from "ioredis";
import { DepartureStatus, VehiclePosition } from "./types";
import connection from "./connection";
import { DataTypeOIDs } from "postgrejs";

const locationPrepared = await connection.prepare(
  `INSERT INTO "vehiclePosition"(realtime_trip_id, lat, lon, speed, heading, received)
     VALUES ($1, $2, $3, $4, $5, $6)
     ON CONFLICT(realtime_trip_id)
       DO UPDATE SET lat      = EXCLUDED.lat,
                     lon      = EXCLUDED.lon,
                     speed    = EXCLUDED.speed,
                     heading  = EXCLUDED.heading,
                     received = EXCLUDED.received;`,
  {
    paramTypes: [
      DataTypeOIDs.text,
      DataTypeOIDs.float8,
      DataTypeOIDs.float8,
      DataTypeOIDs.float8,
      DataTypeOIDs.float8,
      DataTypeOIDs.timestamptz,
    ],
  },
);

const departureStatusPrepared = await connection.prepare(
  `INSERT INTO "departureStatus"(realtime_trip_id, "stopId", "referNext", received)
     VALUES ($1, $2, $3, $4)
     ON CONFLICT(realtime_trip_id)
       DO UPDATE SET "stopId"    = EXCLUDED."stopId",
                     "referNext" = EXCLUDED."referNext",
                     received    = EXCLUDED.received;`,
  {
    paramTypes: [
      DataTypeOIDs.text,
      DataTypeOIDs.int4,
      DataTypeOIDs.bool,
      DataTypeOIDs.timestamptz,
    ],
  },
);
async function onLocation(data: VehiclePosition) {
  const date = new Date(data.received * 1000);
  await locationPrepared.execute({
    params: [
      data.realtime_trip_id,
      data.lat,
      data.lon,
      data.speed,
      data.heading,
      date,
    ],
  });
}

async function onDepartureStatus(data: DepartureStatus) {
  const date = new Date(data.received * 1000);
  await departureStatusPrepared.execute({
    params: [data.realtime_trip_id, data.stopId, data.referNext, date],
  });
}

export async function startListener(redisUrl: string) {
  const subscriber = new Redis(redisUrl);
  subscriber.subscribe("vehiclePosition");
  subscriber.subscribe("departureStatus");
  subscriber.on("message", async (channel, message) => {
    if (channel === "vehiclePosition") {
      await onLocation(JSON.parse(message));
    } else if (channel === "departureStatus") {
      await onDepartureStatus(JSON.parse(message));
    } else throw new Error("unexpected channel'" + channel + "'");
  });
}
